package br.com.nubank.jetty;

import br.com.nubank.keyper.KCertificate;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.HandlerWrapper;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;

public class CertificateRevocationHandler extends HandlerWrapper {
    protected volatile static ConcurrentHashMap<String, Date> crl;

    static {
        try {
            CRLUpdater.updateCRL();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        CRLUpdater.startUpdateCrlScheduler();
    }

    public static void setCrl(ConcurrentHashMap<String, Date> crlHashMap) {
        crl = crlHashMap;
    }

    private X509Certificate findNubankCertificate(X509Certificate[] certs) {
        if (certs != null) {
            for (X509Certificate cert : certs) {
                if ("tcc-client-auth".equals(new KCertificate(cert).getIssuerO())) {
                    return cert;
                }
            }
        }
        return null;
    }

    @Override
    public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        try {
            X509Certificate cert = findNubankCertificate(((X509Certificate[]) request.getAttribute("javax.servlet.request.X509Certificate")));
            if (cert != null) {
                String subject = new KCertificate(cert).getSubjectCN();
                Date certDateIssued = cert.getNotBefore();
                if (crl.containsKey(subject)) {
                    Date crlDate = crl.get(subject);
                    if (certDateIssued.before(crlDate)) {
                        response.setStatus(401);
                        response.setContentType("application/json");
                        response.getOutputStream().write("{ \"error\":\"revoked_certificate\" }".getBytes());
                        baseRequest.setHandled(true);
                    }
                }
            }
        } finally {
            super.handle(target, baseRequest, request, response);
        }
    }
}
