package br.com.nubank.jetty;

import org.eclipse.jetty.util.resource.Resource;
import org.eclipse.jetty.util.ssl.SslContextFactory;

import java.io.InputStream;
import java.security.KeyStore;

public class TLSContextFactory extends SslContextFactory {
    private final String s3KeyStore = "keystore/keystore.p12";
    private final String s3TrustStore = "truststore/truststore.p12";
    private final String password = "uselesspw";

    public TLSContextFactory() {
        setKeyStorePath("");
        setKeyStorePassword(password);
        setWantClientAuth(true);
        setIncludeProtocols("TLSv1.2", "TLSv1.1", "TLSv1");
        setIncludeCipherSuites(
                "TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256",
                "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256",
                "TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384",
                "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384",
                "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA",
                "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA");
        setRenegotiationAllowed(false);
        setExcludeCipherSuites();
    }

    protected KeyStore loadKeyStore() throws Exception {
        return loadKeyStore(s3KeyStore);
    }

    protected KeyStore loadTrustStore() throws Exception {
        return loadKeyStore(s3TrustStore);
    }

    protected KeyStore loadKeyStore(Resource resource) throws Exception {
        return loadKeyStore();
    }

    protected KeyStore loadTrustStore(Resource resource) throws Exception {
        return loadTrustStore();
    }

    protected KeyStore loadKeyStore(String file) throws Exception {
        KeyStore keystore = KeyStore.getInstance("PKCS12");
        S3FileRetriever s3FileRetriever = new S3FileRetriever();
        InputStream keyStoreInputStream = s3FileRetriever.getS3FileInputStream(file);
        keystore.load(keyStoreInputStream, password.toCharArray());
        return keystore;
    }
}
