package br.com.nubank.keyper;

import org.bouncycastle.asn1.ASN1ObjectIdentifier;
import org.bouncycastle.asn1.x500.RDN;
import org.bouncycastle.asn1.x500.X500Name;

import java.security.cert.X509Certificate;
import java.util.Base64;

public class KCertificate {
    private final X509Certificate certificate;

    public KCertificate(X509Certificate certificate) {
        this.certificate = certificate;
    }

    public X509Certificate getCertificate() {
        return this.certificate;
    }

    public String toPem() {
        try {
            byte[] certificateBytes = certificate.getEncoded();
            String header = "-----BEGIN CERTIFICATE-----\n";
            String footer = "\n-----END CERTIFICATE-----";
            String base64Cert = new String(Base64.getEncoder().encode(certificateBytes));
            return header + base64Cert + footer;
        } catch (Exception e) {
            throw new KeyperException(e);
        }
    }

    public String getIssuerO() {
        X500Name issuerX500Name = new X500Name(this.certificate.getIssuerX500Principal().getName());
        return ((RDN[]) issuerX500Name.getRDNs(new ASN1ObjectIdentifier("2.5.4.10")))[0].getTypesAndValues()[0].getValue().toString();
    }

    public String getIssuerCN() {
        X500Name issuerX500Name = new X500Name(this.certificate.getIssuerX500Principal().getName());
        return ((RDN[]) issuerX500Name.getRDNs(new ASN1ObjectIdentifier("2.5.4.3")))[0].getTypesAndValues()[0].getValue().toString();
    }

    public String getSubjectCN() {
        X500Name subjectX500Name = new X500Name(this.certificate.getSubjectX500Principal().getName());
        return ((RDN[]) subjectX500Name.getRDNs(new ASN1ObjectIdentifier("2.5.4.3")))[0].getTypesAndValues()[0].getValue().toString();
    }
}
