(ns service.server
  (:gen-class)
  (:require [io.pedestal.http :as server]
            [io.pedestal.http.route :as route]
            [service.service :as service]))

(defonce runnable-service (server/create-server service/service))

(defn run-dev
  "The entry-point for 'lein run-dev'"
  [& args]
  (println "\nCreating your [DEV] server...")
  (-> service/service
      (merge {:env :dev
              ::server/join? false
              ::server/routes #(route/expand-routes (deref #'service/routes))
              ::server/allowed-origins {:creds true :allowed-origins (constantly true)}})
      server/default-interceptors
      server/dev-interceptors
      server/create-server
      server/start))

(defn -main
  "The entry-point for 'lein run'"
  [& args]
  (println "\nCreating your server...")
  (server/start runnable-service))

(defonce servlet  (atom nil))

(defn servlet-init
  [_ config]
  (reset! servlet  (server/servlet-init service/service nil)))

(defn servlet-service
  [_ request response]
  (server/servlet-service @servlet request response))

(defn servlet-destroy
  [_]
  (server/servlet-destroy @servlet)
  (reset! servlet nil))
